@@ -1,5 +1,5 @@
 g.readaccfile = function(filename,blocksize,blocknumber,selectdaysfile=c(),filequality,
-                         decn,dayborder,ws, desiredtz = c()) {
+                         decn,dayborder,ws, desiredtz = c(), PreviousEndPage = 1,inspectfileobject=c()) {
   # function wrapper to read blocks of accelerationd data from various brands
   # the code identifies which accelerometer brand and data format it is
   # blocksize = number of pages to read at once
@@ -15,26 +15,48 @@ g.readaccfile = function(filename,blocksize,blocknumber,selectdaysfile=c(),fileq
   # sf = sample frequency (Hertz)
   # ws = large window size (default 3600 seconds)
   switchoffLD = 0
-  
   if (length(unlist(strsplit(filename,"[.]RD"))) > 1) {
     useRDA = TRUE
   } else {
     useRDA = FALSE
   }
   if (useRDA == FALSE) {
-    I = g.inspectfile(filename) ## modified by JH
+    I = inspectfileobject
     mon = I$monc
     dformat = I$dformc
     sf = I$sf
   }
   P = c()
-  
+  updatepageindexing = function(startpage=c(), deltapage=c(), blocknumber=c(),PreviousEndPage=c(),
+                                mon=c(),dformat=c()) {
+    # This function ensures that startpage is only specified for blocknumber 1.
+    # The next time (blocknumber > 1) the startpage will be derived from the previous
+    # endpage and the blocksize.
+    if (blocknumber != 1 & length(PreviousEndPage) != 0) {
+      if ((mon == 2 & dformat == 1) | dformat == 2) {
+        # only in GENEActiv binary data and for csv format data
+        # page selection is defined from start to end (including end)
+        startpage = PreviousEndPage + 1
+      } else { 
+        # for other monitor brands and data formats
+        # page selection is defined from start to end (excluding end itself)
+        # so start page of one block equals the end page of previous block
+        startpage = PreviousEndPage
+      }
+    }
+    endpage = startpage + deltapage
+    return(list(startpage=startpage,endpage=endpage))
+  }
   if (mon == 1 & dformat == 1) { # genea binary
-    try(expr={P = g.binread(binfile=filename,(blocksize*(blocknumber-1)),(blocksize*blocknumber))},silent=TRUE)
+    startpage = blocksize*(blocknumber-1)
+    deltapage = blocksize
+    UPI = updatepageindexing(startpage=startpage,deltapage=deltapage,
+                             blocknumber=blocknumber,PreviousEndPage=PreviousEndPage, mon=mon, dformat=dformat)
+    startpage = UPI$startpage;    endpage = UPI$endpage
+    try(expr={P = g.binread(binfile=filename,startpage,endpage)},silent=TRUE)
     if (length(P) > 1) {
       if (nrow(P$rawxyz) < ((sf*ws*2)+1) & blocknumber == 1) {
         P = c() ; switchoffLD = 1 #added 30-6-2012
-        cat("\nError: data too short for doing non-wear detection 1\n")		
         filequality$filetooshort = TRUE
       }
     } else {
@@ -42,14 +64,17 @@ g.readaccfile = function(filename,blocksize,blocknumber,selectdaysfile=c(),fileq
       if (blocknumber == 1) {
         filequality$filecorrupt = TRUE
       }
-      cat("\nEnd of file reached\n")
     }
   } else if (mon == 4 & dformat == 3) { # axivity wav
-    try(expr={P = g.wavread(wavfile=filename,(blocksize*(blocknumber-1)),(blocksize*blocknumber))},silent=TRUE)
+    startpage = blocksize*(blocknumber-1)
+    deltapage = blocksize
+    UPI = updatepageindexing(startpage=startpage,deltapage=deltapage,
+                             blocknumber=blocknumber,PreviousEndPage=PreviousEndPage, mon=mon, dformat=dformat)
+    startpage = UPI$startpage;    endpage = UPI$endpage
+    try(expr={P = g.wavread(wavfile=filename,startpage,endpage)},silent=TRUE)
     if (length(P) > 1) {
       if (nrow(P$rawxyz) < ((sf*ws*2)+1) & blocknumber == 1) {
         P = c() ; switchoffLD = 1 #added 30-6-2012
-        cat("\nError: data too short for doing non-wear detection 1\n")		
         filequality$filetooshort = TRUE
       }
     } else {
@@ -57,18 +82,15 @@ g.readaccfile = function(filename,blocksize,blocknumber,selectdaysfile=c(),fileq
       if (blocknumber == 1) {
         filequality$filecorrupt = TRUE
       }
-      cat("\nEnd of file reached\n")
     }
   } else if (mon == 2 & dformat == 1 & useRDA == FALSE) { # GENEActiv binary non-RDA format
     if (length(selectdaysfile) > 0) { # code to only read fragments of the data (Millenium cohort)
       #===================================================================
       # All of the below needed for Millenium cohort
       SDF = read.csv(selectdaysfile, stringsAsFactors = FALSE) # small change by CLS
-      
       hvars = g.extractheadervars(I)
       SN = hvars$SN
       SDFi = which(basename(SDF$binFile) == basename(filename))
-      
       if(length(SDFi) != 1) {
         save(SDF, SDFi, file = "debuggingFile.Rda")
         stop(paste0("CLS error: there are zero or more than one files: ",
@@ -80,7 +102,7 @@ g.readaccfile = function(filename,blocksize,blocknumber,selectdaysfile=c(),fileq
       hhr <- GENEAread::header.info(filename)
       tint <- rbind(getStartEndNumeric(SDF$Day1[SDFi], hhr = hhr, startHour = dayborder),
                     getStartEndNumeric(SDF$Day2[SDFi], hhr = hhr, startHour = dayborder))
-      
+
       if (blocknumber == nrow(tint)+1 | nrow(tint) == 0) {
         #all data read now make sure that it does not try to re-read it with mmap on
         switchoffLD = 1
@@ -90,28 +112,18 @@ g.readaccfile = function(filename,blocksize,blocknumber,selectdaysfile=c(),fileq
                                   end=tint[blocknumber,2],calibrate=TRUE,do.temp=TRUE,mmap.load=FALSE)
           if (sf != P$freq) sf = P$freq
         },silent=TRUE)
-        
-        # llll
         if (length(P) <= 2) {
-          cat("\ninitial attempt to read data unsuccessful, try again with mmap turned on:\n")
           #try again but now with mmap.load turned on
-          if (length(P) != 0) {
-            cat("\ndata read succesfully\n")
-          } else {
-            switchoffLD = 1
-          }
+          if (length(P) == 0) switchoffLD = 1
         }
       }
-      ############################
       if (length(P) > 0) {
-        if (length(selectdaysfile) > 0) { 
+        if (length(selectdaysfile) > 0) {
           if (tint[blocknumber,1] == "0") {
-            print("last block")
             switchoffLD = 1
           }
         } else {
           if (nrow(P$data.out) < (blocksize*300)) { #last block
-            print("last block")
             switchoffLD = 1
           }
         }
@@ -130,7 +142,6 @@ g.readaccfile = function(filename,blocksize,blocknumber,selectdaysfile=c(),fileq
           }
         } else {
           P= c() #just no data in this last block
-          # cat("\nnot enough data in this block 3\n")
         }
       } else { #check whether there is enough data
         if (nrow(P$data.out) < ((sf*ws*2)+1) & blocknumber == 1) {
@@ -142,30 +153,31 @@ g.readaccfile = function(filename,blocksize,blocknumber,selectdaysfile=c(),fileq
       }
       # All of the above needed for Millenium cohort
       #======================================================================
-    } else { 
-      try(expr={P = GENEAread::read.bin(binfile=filename,start=(blocksize*(blocknumber-1)),
-                                        end=(blocksize*blocknumber),calibrate=TRUE,do.temp=TRUE,mmap.load=FALSE)},silent=TRUE)
+    } else {
+      startpage = blocksize*(blocknumber-1) + 1 # GENEActiv starts with page 1
+      deltapage = blocksize
+      UPI = updatepageindexing(startpage=startpage,deltapage=deltapage,
+                               blocknumber=blocknumber,PreviousEndPage=PreviousEndPage, mon=mon, dformat=dformat)
+      startpage = UPI$startpage;    endpage = UPI$endpage
+      try(expr={P = GENEAread::read.bin(binfile=filename,start=startpage,
+                                        end=endpage,calibrate=TRUE,do.temp=TRUE,mmap.load=FALSE)},silent=TRUE)
       if (length(P) <= 2) {
-        cat("\ninitial attempt to read data unsuccessful, try again with mmap turned on:\n")
         #try again but now with mmap.load turned on
-        try(expr={P = GENEAread::read.bin(binfile=filename,start=(blocksize*(blocknumber-1)),
-                                          end=(blocksize*blocknumber),calibrate=TRUE,do.temp=TRUE,mmap.load=TRUE)},silent=TRUE)
+        try(expr={P = GENEAread::read.bin(binfile=filename,start=startpage,
+                                          end=endpage,calibrate=TRUE,do.temp=TRUE,mmap.load=TRUE)},silent=TRUE)
         if (length(P) != 0) {
-          cat("\ndata read succesfully\n")
           if (sf != P$freq) sf = P$freq
         } else {
           switchoffLD = 1
         }
       }
       if (length(P) > 0) {
-        if (length(selectdaysfile) > 0) { 
+        if (length(selectdaysfile) > 0) {
           if (tint[blocknumber,1] == "0") {
-            print("last block")
             switchoffLD = 1
           }
         } else {
           if (nrow(P$data.out) < (blocksize*300)) { #last block
-            print("last block")
             switchoffLD = 1
           }
         }
@@ -181,81 +193,85 @@ g.readaccfile = function(filename,blocksize,blocknumber,selectdaysfile=c(),fileq
           } #if not then P is now filled with data
         } else {
           P= c() #just no data in this last block
-          # cat("\nnot enough data in this block 3\n")
         }
       }
       if (length(P) > 0) { #check whether there is enough data
         if (nrow(P$data.out) < ((sf*ws*2)+1) & blocknumber == 1) {
           P = c();  switchoffLD = 1
-          cat("\nWarning (2): data in block too short for doing non-wear detection\n")
           filequality$filetooshort = TRUE
         }
       }
     }
     #===============
   } else if (mon == 2 & dformat == 2) { # GENEActiv csv format
-    cat("\nGeneactiv in csv-format\n")
-    try(expr={P = read.csv(filename,nrow = (blocksize*300), skip=(100+(blocksize*300*(blocknumber-1))),header = FALSE,dec=decn)},silent=TRUE)
+    startpage = (100+(blocksize*300*(blocknumber-1)))
+    deltapage = (blocksize*300)
+    UPI = updatepageindexing(startpage=startpage,deltapage=deltapage,
+                             blocknumber=blocknumber,PreviousEndPage=PreviousEndPage, mon=mon, dformat=dformat)
+    startpage = UPI$startpage;    endpage = UPI$endpage
+    try(expr={P = read.csv(filename,nrow = deltapage, skip=startpage,header = FALSE,dec=decn)},silent=TRUE)
     if (length(P) > 1) {
       P = as.matrix(P)
       if (nrow(P) < ((sf*ws*2)+1) & blocknumber == 1) {
         P = c() ; switchoffLD = 1 #added 30-6-2012
-        cat("\nWarning (1): data in block too short for doing non-wear detection\n")		
         filequality$filetooshort = TRUE
       }
     } else {
       P = c()
-      cat("\nEnd of file reached\n")
     }
   } else if (mon == 3 & dformat == 2) { # Actigraph csv format
     headerlength = 10
     #--------------
-    skiprows = (headerlength+(blocksize*300*(blocknumber-1)))
+    startpage = (headerlength+(blocksize*300*(blocknumber-1)))
+    deltapage = blocksize*300
+    UPI = updatepageindexing(startpage=startpage,deltapage=deltapage,
+                             blocknumber=blocknumber,PreviousEndPage=PreviousEndPage, mon=mon, dformat=dformat)
+    startpage = UPI$startpage;    endpage = UPI$endpage
     # load rows 11:13  to investigate whether the file has a header
-    testheader = as.data.frame(data.table::fread(filename,nrow = 2, 
-                                                 skip=10, 
-                                                 dec=",",showProgress = FALSE, header = FALSE))
+    testheader = as.data.frame(data.table::fread(filename,nrow = 2,
+                                                 skip=10,
+                                                 dec=decn,showProgress = FALSE, header = FALSE))
     if (suppressWarnings(is.na(as.numeric(testheader[1,1]))) ==  FALSE) { # it has no header, first value is a number
       freadheader = FALSE
     } else { # it has a header, first value is a character
       freadheader = TRUE
       headerlength = 11
-      if (skiprows == 10) {
-        skiprows = 11
+      if (startpage == 10) {
+        startpage = 11
         freadheader = FALSE
       }
     }
     #--------------
     try(expr={
       P = as.data.frame(
-        data.table::fread(filename,nrow = (blocksize*300), 
-                          skip=skiprows, 
+        data.table::fread(filename,nrow = deltapage,
+                          skip=startpage,
                           dec=decn,showProgress = FALSE, header = freadheader))
     },silent=TRUE)
     if (length(P) > 1) {
       P = as.matrix(P)
       if (nrow(P) < ((sf*ws*2)+1) & blocknumber == 1) {
         P = c() ; switchoffLD = 1 #added 30-6-2012
-        cat("\nWarning (1): data in block too short for doing non-wear detection\n")
         filequality$filetooshort = TRUE
       }
     } else {
       P = c()
-      cat("\nEnd of file reached\n")
     }
   } else if (mon == 4 & dformat == 4) { # axivity cwa
-    try(expr={P = g.cwaread(fileName=filename, start = (blocksize*(blocknumber-1)), # try to read block first time
-                            end = (blocksize*blocknumber), progressBar = FALSE)},silent=TRUE)
-    
+    startpage = blocksize*(blocknumber-1)
+    deltapage = blocksize
+    UPI = updatepageindexing(startpage=startpage,deltapage=deltapage,
+                             blocknumber=blocknumber,PreviousEndPage=PreviousEndPage, mon=mon, dformat=dformat)
+    startpage = UPI$startpage;    endpage = UPI$endpage
+    try(expr={P = g.cwaread(fileName=filename, start = startpage, # try to read block first time
+                            end = endpage, progressBar = FALSE, desiredtz = desiredtz)},silent=TRUE)
     if (length(P) > 1) { # data reading succesful
       if (length(P$data) == 0) { # too short?
         P = c() ; switchoffLD = 1
-        cat("\nWarning (1): Data in block too short for doing non-wear detection\n")
         if (blocknumber == 1) filequality$filetooshort = TRUE
-      } else { # too short for non-wear detection
+      } else {
         if (nrow(P$data) < ((sf*ws*2)+1)) {
           P = c() ; switchoffLD = 1
-          cat("\nError: Data too short for doing non-wear detection 1\n")		
           if (blocknumber == 1) filequality$filetooshort = TRUE
         }
       }
@@ -263,77 +279,69 @@ g.readaccfile = function(filename,blocksize,blocknumber,selectdaysfile=c(),fileq
       # Now only load the last page, to assess whether there may be something wrong with this block of data:
       # I (VvH) implemented this as a temporary fix on 17Nov2018, but it would be better if we understood the source of this error
       # and address it inside the g.cwaread function. For example, are the page corrupted, and if so then why?
-      PtestLastPage = c()
-      # try to read the last page of the block
-      try(expr={PtestLastPage = g.cwaread(fileName=filename, start = (blocksize*blocknumber),
-                                          end = (blocksize*blocknumber), progressBar = FALSE)},silent=TRUE)
+      PtestLastPage = PtestStartPage = c()
+      # try to read the last page of the block, because if it exists then there might be something wrong with the first page(s).
+      try(expr={PtestLastPage = g.cwaread(fileName=filename, start = endpage, #note this is intentionally endpage
+                                          end = endpage, progressBar = FALSE, desiredtz = desiredtz)},silent=TRUE)
       if (length(PtestLastPage) > 1) { # Last page exist, so there must be something wrong with the first page
         NFilePagesSkipped = 0
-        PtestStartPage = c()
         while (length(PtestStartPage) == 0) { # Try loading the first page of the block by iteratively skipping a page
-          NFilePagesSkipped = NFilePagesSkipped + 1 
-          try(expr={PtestStartPage = g.cwaread(fileName=filename, start = (blocksize*(blocknumber-1)) + NFilePagesSkipped,
-                                               end = (blocksize*(blocknumber-1)) + NFilePagesSkipped, progressBar = FALSE)},silent=TRUE)
+          NFilePagesSkipped = NFilePagesSkipped + 1
+          startpage = startpage + NFilePagesSkipped
+          try(expr={PtestStartPage = g.cwaread(fileName=filename, start = startpage , # note: end is intentionally startpage
+                                               end = startpage, progressBar = FALSE, desiredtz = desiredtz)},silent=TRUE)
           if (NFilePagesSkipped == 10 & length(PtestStartPage) == 0) PtestStartPage = FALSE # stop after 10 attempts
         }
         cat(paste0("\nWarning (4): ",NFilePagesSkipped," page(s) skipped in cwa file in order to read data-block, this may indicate data corruption."))
       }
-      if (length(PtestStartPage) > 1) { 
+      if (length(PtestStartPage) > 1) {
         # Now we know on which page we can start and end the block, we can try again to
         # read the entire block:
-        try(expr={P = g.cwaread(fileName=filename, start = (blocksize*(blocknumber-1))+NFilePagesSkipped,
-                                end = (blocksize*blocknumber), progressBar = FALSE)},silent=TRUE)
+        try(expr={P = g.cwaread(fileName=filename, start = startpage,
+                                end = endpage, progressBar = FALSE, desiredtz = desiredtz)},silent=TRUE)
         if (length(P) > 1) { # data reading succesful
           if (length(P$data) == 0) { # if this still does not work then
             P = c() ; switchoffLD = 1
-            cat("\nWarning (3): data in block too short for doing non-wear detection\n")
             if (blocknumber == 1) filequality$filetooshort = TRUE
           } else {
             if (nrow(P$data) < ((sf*ws*2)+1)) {
               P = c() ; switchoffLD = 1
-              cat("\nError: data too short for doing non-wear detection 1\n")		
               if (blocknumber == 1) filequality$filetooshort = TRUE
             } else {
               filequality$NFilePagesSkipped = NFilePagesSkipped # store number of pages jumped
             }
           }
           # Add replications of Ptest to the beginning of P to achieve same data length as under nuormal conditions
-          P$data = rbind(do.call("rbind",replicate(NFilePagesSkipped,PtestStartPage$data,simplify = FALSE)), P$data) 
-          
+          P$data = rbind(do.call("rbind",replicate(NFilePagesSkipped,PtestStartPage$data,simplify = FALSE)), P$data)
         } else { # Data reading still not succesful, so classify file as corrupt
           P = c()
-          if (blocknumber == 1) {
-            filequality$filecorrupt = TRUE
-          }
-          cat("\nEnd of file reached\n")
+          if (blocknumber == 1) filequality$filecorrupt = TRUE
         }
-      } else { 
+      } else {
         P = c()
-        if (blocknumber == 1) {
-          filequality$filecorrupt = TRUE
-        }
-        cat("\nEnd of file reached\n")
+        if (blocknumber == 1) filequality$filecorrupt = TRUE
       }
     }
   } else if (mon == 4 & dformat == 2) { # axivity (ax3) csv format
     freadheader = FALSE
     headerlength = 0
-    skiprows = (headerlength+(blocksize*300*(blocknumber-1)))
+    startpage = (headerlength+(blocksize*300*(blocknumber-1)))
+    deltapage = (blocksize*300)
+    UPI = updatepageindexing(startpage=startpage,deltapage=deltapage,
+                             blocknumber=blocknumber,PreviousEndPage=PreviousEndPage, mon=mon, dformat=dformat)
+    startpage = UPI$startpage;    endpage = UPI$endpage
     try(expr={
       P = as.data.frame(
-        data.table::fread(filename,nrow = (blocksize*300), 
-                          skip=skiprows, 
+        data.table::fread(filename,nrow = deltapage,
+                          skip=startpage,
                           dec=decn,showProgress = FALSE, header = freadheader))
     },silent=TRUE)
     if (length(P) > 1) {
-      # P = as.matrix(P)
       if (nrow(P) < ((sf*ws*2)+1) & blocknumber == 1) {
         P = c() ; switchoffLD = 1 #added 30-6-2012
-        cat("\nWarning (1): data in block too short for doing non-wear detection\n")
         filequality$filetooshort = TRUE
       }
-      if (nrow(P) < (blocksize*300)) { #last block
-        print("last block")
+      if (nrow(P) < (deltapage)) { #last block
         switchoffLD = 1
       }
       # resample the acceleration data, because AX3 data is stored at irregular time points
@@ -358,8 +366,7 @@ g.readaccfile = function(filename,blocksize,blocknumber,selectdaysfile=c(),fileq
       P = cbind(timeRes,accelRes)
     } else {
       P = c()
-      cat("\nEnd of file reached\n")
     }
   }
-  invisible(list(P=P,filequality=filequality, switchoffLD = switchoffLD))
+  invisible(list(P=P,filequality=filequality, switchoffLD = switchoffLD, endpage = endpage))
 }
